# -*- coding: utf-8 -*-

__author__ = 'Guilherme Castelao'
__email__ = 'guilherme@castelao.net'
__version__ = '0.11.0'

from .cnv import CNV, fCNV
from .exceptions import CNVError

# __all__ = ['CNV', 'fCNV']
